<?php
// Page prototype, other page types are based on (extended from) this abstract class.
abstract class Page
{
    protected $cfg;
    protected $db;
    protected $user;

    protected $action;
    protected $showOutput; // whether to display header and footer

    protected $title;
    protected $output; // page type output

    public function __construct($options) {
        $this->cfg      = $options['config'];
        $this->db       = $options['database'];
        $this->user     = $options['user'];
        $this->route    = $options['routing'];

        if (isset($_GET['action']) && method_exists($this, 'action'.ucfirst($_GET['action']))) {
            $this->action = $_GET['action'];
        } else {
            $this->action = 'default';
        }
        $this->showOutput = true; // whether to render an entire HTML document (header + content + footer) or just the content
        $this->title = '';
    }

    protected function __header() {
?><!DOCTYPE html>
<html>
<head>
<meta charset="UTF-8">
<title><?php echo $this->escape($this->title) ?></title>
<base href="<?php echo $this->escape($this->link()) ?>">
<link rel="stylesheet" type="text/css" href="media/css/default.css" media="screen">
</head>

<body><?php
    }

    // Usually this part contains not just the content but the general layout of the page as well.
    // There is something to say to split the Page class into a PageType class (solely for the content)
    // and a Maintemplate class (specifically for the general page layout, in which a PageType only
    // forms a (single/small) part).
    protected function __main() {
        echo $this->output;
    }

    protected function __footer() {
?></body>
</html><?php
    }

    public function exec() {
        // Defaults - can be overridden in page init or action.
        header('HTTP/1.0 200 OK');
        header('Content-Type: text/html; charset=UTF-8');

        $action = 'action'.ucfirst($this->action);
        $this->init();

        // Distinguish between whether we do or do not produce output.
        if ($this->showOutput) {
            ob_start(); // output buffer for content
                // Execute Page action first.
                // This is done so the action method can pass information to the header or footer of the page.
                // For example: you can set a page title, metadata or include CSS or JS files, or you could
                // add inline JavaScript to the end of your page. You can write additional code in the Page class
                // (or extend from the original) to accommodate for this.
                $this->$action();
            $this->output = ob_get_clean(); // catch content

            // Print entire page in the right order: header - content - footer
            $this->__header();
                $this->__main();
            $this->__footer();
        } else {
            // Just execute page action.
            $this->$action();
        }
    }

    // Optional method.
    protected function init() {}

    // Required method, to be implemented in your own page types.
    abstract protected function actionDefault();

    protected function setTitle($title) {
        $this->title = $title;
    }

    // Shorthand (alias).
    protected function escape($in) {
        return Helper::escape($in);
    }

    // Shorthand (alias).
    protected function redirect($in) {
        return $this->route->redirect($in);
    }

    // Shorthand (alias).
    protected function link($path=false, $args=array(), $options=array()) {
        return $this->route->link($path, $args, $options);
    }
}