<?php
class Page_Login extends Page
{
    protected function init() {
        // Define actions without output.
        if (in_array($this->action, array(
            'login',
            'logout',
        ))) {
            $this->showOutput = false;
        }
    }

    protected function actionDefault() {
        $this->setTitle('Log in');

        ?><h1>Log in</h1><?php
        if ($this->user->isLoggedIn()) {
            ?><p>You are already logged in.</p><?php
        } else {
            if (isset($_GET['errors'])) {
                ?><p class="attention">Invalid user name or password, try again.</p><?php
            }
            $formAction = $this->link(false, array('action' => 'login'));
            $token = Helper::getRandomToken();
            $_SESSION['form']['token'] = $token;
            ?><div class="form">
                <form action="<?php echo $this->escape($formAction) ?>" method="post" accept-charset="UTF-8">
                    <div>
                        <input type="hidden" name="token" value="<?php echo $this->escape($token) ?>" />
                        <p><label for="username">user name</label><input type="text" name="username" id="username" value="" autocomplete="off" /></p>
                    </div>
                    <div>
                        <p><label for="password">password</label><input type="password" name="password" id="password" value="" /></p>
                    </div>
                    <div class="buttons">
                        <button type="submit">login</button>
                    </div>
                </form>
            </div><?php
        }
    }

    protected function actionLogin() {
        if ($_SERVER['REQUEST_METHOD'] == 'POST') {
            // check CSRF token
            if (
                empty($_SESSION['form']['token']) ||
                empty($_POST['token']) ||
                $_SESSION['form']['token'] !== $_POST['token']
            ) {
                throw new Exception('Login: invalid form token');
            }

            $errors = false;
            // @todo check existence of fields first... for now we assume they exist

            // username and password are required, use the same checks as with registration
            // note that we do not disclose WHAT is wrong, we simply report back that SOMETHING is wrong
            // username - check valid username
            if (User::isValidUserName(trim($_POST['username'])) === false) {
                $errors = true;
            }

            // password - just check length for now
            if (strlen($_POST['password']) < 6) {
                $errors = true;
            }

            if ($errors) {
                // do not even bother with an authentication attempt
                // do nothing (we will be redirected at the end)
            } else {
                $userId = $this->user->authenticate($_POST['username'], $_POST['password']);
                if ($userId === false) {
                    // bad credentials, do nothing (we will be redirected at the end)
                } else {
                    // login succesful, further handle the login process
                    $this->user->login($userId);
                    // redirect the user to a "success" page
                    $this->redirect($this->link(false, array('action' => 'success')));
                }
            }
        }
        // If we reached this point, something went wrong while processsing login data.
        $this->redirect($this->link(false, array('errors' => true)));
    }

    protected function actionSuccess() {
        ?><h1>Hi <?php echo $this->escape($this->user->name) ?>!</h1>
        <p>You are now logged in.</p>
        <p><a href="<?php echo $this->escape($this->link($this->cfg->defaultPath)) ?>">to the homepage</a></p><?php
    }

    protected function actionLogout() {
        $this->user->logout();
        $this->redirect($this->link(false));
    }
}