<?php
/*
CREATE TABLE users (
    usr_id              INT(10) UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
    usr_name            VARCHAR(100) NOT NULL,
    usr_password        VARCHAR(64) NOT NULL,
    usr_password_salt   VARCHAR(64) NOT NULL,
    usr_email           VARCHAR(255) NOT NULL
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
*/
class Page_Register extends Page
{
    protected function init() {
        // Define the actions that do not generate output.
        if (in_array($this->action, array(
            'register',
        ))) {
            $this->showOutput = false;
        }
    }

    // helper function
    protected function __printSessionErrors($var) {
        if (isset($_SESSION['form']['errors'][$var])) {
            ?><ul class="errors"><?php
                foreach ($_SESSION['form']['errors'][$var] as $error) {
                    ?><li><?php echo $this->escape($error) ?></li><?php
                }
            ?></ul><?php
        }
    }

    // helper function
    protected function __printSessionValue($var) {
        if (isset($_SESSION['form']['data'][$var])) {
            echo $this->escape(trim($_SESSION['form']['data'][$var]));
        }
    }

    protected function actionDefault() {
        $this->setTitle('Register');

        // echo '<pre>'.print_r($_SESSION, true).'</pre>';

        ?><h1>Register</h1><?php
        if (isset($_GET['errors'])) {
            ?><p class="attention">Errors occurred - please check your input.</p><?php
        } else {
            // reset $_SESSION
            $_SESSION['form'] = array();
        }

        $token = Helper::getRandomToken();
        $_SESSION['form']['token'] = $token;
        $registerLink = $this->link(false, array('action' => 'register'));

        ?><div class="form">
            <form action="<?php echo $this->escape($registerLink) ?>" method="post" accept-charset="UTF-8">
                <div>
                    <input type="hidden" name="token" value="<?php echo $this->escape($token) ?>" />
                    <p><label for="username">user name</label><input type="text" name="username" id="username" value="<?php $this->__printSessionValue('username') ?>" autocomplete="off" /></p><?php
                    $this->__printSessionErrors('username');
                ?></div>
                <div>
                    <p><label for="password">password</label><input type="password" name="password" id="password" value="" /></p><?php
                    $this->__printSessionErrors('password');
                ?></div>
                <div>
                    <p><label for="password2">repeat password</label><input type="password" name="password2" id="password2" value="" /></p><?php
                    $this->__printSessionErrors('password2');
                ?></div>
                <div>
                    <p><label for="email">e-mail address</label><input type="text" name="email" id="email" value="<?php $this->__printSessionValue('email') ?>" autocomplete="off" /></p><?php
                    $this->__printSessionErrors('email');
                ?></div>
                <div class="buttons">
                    <button type="submit">register</button>
                </div>
            </form>
        </div><?php
    }

    protected function actionRegister() {
        if ($_SERVER['REQUEST_METHOD'] == 'POST') {
            $errors = array();

            // validate data
            // we use a transaction to ensure that all queries are performed as a single
            // undividable (atomic) action
            $this->db->beginTransaction();
                // check CSRF token
                if (
                    empty($_SESSION['form']['token']) ||
                    empty($_POST['token']) ||
                    $_SESSION['form']['token'] !== $_POST['token']
                ) {
                    throw new Exception('Register: invalid form token');
                }

                // username - uniqueness (case-insensitive)
                /* @var $st PDOStatement */
                $st = $this->db->prepare(
                    'SELECT usr_id
                    FROM users
                    WHERE usr_name = ?
                    FOR UPDATE'
                );

                // @todo check existence of fields first... for now we assume they exist
                $st->execute(array(trim($_POST['username'])));
                if ($st->rowCount() > 0) {
                    $errors['username'][] = 'user name already exists';
                }
                // username - check valid username
                if (User::isValidUserName(trim($_POST['username'])) === false) {
                    $errors['username'][] = 'invalid username';
                }

                // password - just check length for now
                if (strlen($_POST['password']) < 6) {
                    $errors['password'][] = 'password must consist of at least 6 characters';
                }
                // password - equality
                if (strcmp($_POST['password'], $_POST['password2']) !== 0) {
                    $errors['password'][] = 'passwords are not equal';
                    $errors['password2'][] = 'passwords are not equal';
                }

                // e-mail - uniqueness (case-insensitive)
                /* @var $st PDOStatement */
                $st = $this->db->prepare(
                    'SELECT usr_id
                    FROM users
                    WHERE usr_email = ?
                    FOR UPDATE'
                );
                $st->execute(array(trim($_POST['email'])));
                if ($st->rowCount() > 0) {
                    $errors['email'][] = 'e-mail address already in use';
                }

                // e-mail - validity (filter_var works for most cases, but is not infallible)
                if (filter_var(trim($_POST['email']), FILTER_VALIDATE_EMAIL) === false) {
                    $errors['email'][] = 'e-mail address does not validate';
                } else {
                    // optional - check MX
                    $email = trim($_POST['email']);
                    $domain = substr($email, strpos($email, '@') + 1);
                    if (checkdnsrr($domain, 'MX') === false) {
                        $errors['email'][] = 'no MX record found for domain of e-mail address';
                    }
                }

                // redirect on errors
                if (count($errors) > 0) {
                    $_SESSION['form'] = array(
                        // store previously submitted data
                        'data' => array(
                            'username'  => trim($_POST['username']),
                            'email'     => trim($_POST['email']),
                        ),
                        'errors' => $errors,
                    );
                    $this->redirect($this->link(false, array('errors' => true)));
                }

                // if we reach this part, all data was successfully validated and can be processed
                $password = User::createPassword($_POST['password'], Helper::getRandomToken());
                /* @var $st PDOStatement */
                $st = $this->db->prepare(
                    'INSERT INTO users(
                        usr_name, usr_password, usr_password_salt, usr_email
                    ) VALUES (?, ?, ?, ?)'
                );
                $st->execute(array(
                    trim($_POST['username']),
                    $password['password'],
                    $password['salt'],
                    trim($_POST['email']),
                ));
            $this->db->commit();

            // redirect
            $this->redirect($this->link(false, array('action' => 'success')));
        }
        $this->redirect($this->link());
    }

    protected function actionSuccess() {
        ?><h1>Success!</h1>
        <p>Your registration was succesful.</p>
        <p>You can now <a href="<?php echo $this->escape($this->link('login')) ?>">log in</a>.</p><?php
    }
}