<?php
class Routing
{
    protected $cfg;
    protected $db;

    protected $path; // path relative to $this->cfg->relativeURL

    public function __construct($options) {
        $this->cfg   = $options['config'];
        $this->db    = $options['database'];
        $this->path  = '';
    }

    public function init() {
        // Calculate the relative (cleaned) path.
        $urlParts = @parse_url($_SERVER['REQUEST_URI']);
        if ($urlParts === false) {
            throw new Exception('Routing: could not parse URL');
            // @todo alternative: serve 404?
        }
        $path = trim($urlParts['path'], '/'); // strip leading and trailing slashes
        if (strlen($this->cfg->relativeURL) > 0) {
            $path = substr($path, strlen($this->cfg->relativeURL));
        }
        if ($path == 'index.php' || $path == '') {
            // Alternatively, you could redirect /index.php to / with:
            // $this->redirect($this->link());
            $path = $this->cfg->defaultPath;
        }
        $this->path = $path;
    }

    public function getPath() {
        return $this->path;
    }

    // Calculates page (class) to be loaded.
    public function handleRequest() {
        // Check if a class associated with $this->path exists
        $class = 'Page_'.implode('_', array_map('ucfirst', explode('/', $this->path)));
        // class_exists() tries to load the class through the autoloader automatically.
        // So basically we are doing a file_exists() + a check to see if the file contains
        // a valid class name (class_exists()) in one go.
        if (class_exists($class) === false) {
            $class = 'Page_Error_404';
        }
        // Return the calculated class name.
        return $class;
    }

    public function redirect($link) {
        header('HTTP/1.1 303 See Other');
        header('Location: '.$link);
        exit; // so we never forget :)
    }

    // Note: options not used currently.
    public function link($path=false, $args=array(), $options=array()) {
        // @todo add secure option? perhaps through $this->cfg
        if ($path === false) {
            $path = $this->path; // take current page
        }
        if ($path === $this->cfg->defaultPath) {
            $path = ''; // make path empty
        }
        $link = 'http://'.$_SERVER['HTTP_HOST'].$this->cfg->relativeURL.'/'.$path;

        if (count($args) > 0) {
            $separator = '?';
            foreach ($args as $k => $v) {
                $link .= $separator.urlencode($k).'='.urlencode($v);
                $separator = '&';
            }
        }
        return $link;
    }
}